import { ElementRef, EventEmitter, OnChanges, OnInit, SimpleChanges } from '@angular/core';
import { Annotation } from '../../models/annotation.model';
import { ISelection } from '../../models/selection.model';
import { TokenizerService } from '../../services/tokenizer.service';
import * as i0 from "@angular/core";
export declare class NgxAnnotateTextComponent implements OnInit, OnChanges {
    private elementRef;
    private tokenService;
    /** Represents the parts of the given text which shall be annotated. */
    annotations: Annotation[];
    /** An optional CSS class applied to all elements which wrap the annotated parts of the given text. */
    annotationClass?: string;
    /**
     * Determines whether annotations shall have a small button in the top right corner so that the user can
     * remove an annotation.
     */
    removable: boolean;
    /** The text which shall be displayed and annotated. */
    text: string;
    /** Emits the list of existing annotations after an element has been removed. */
    annotationsChange: EventEmitter<Annotation[]>;
    private selectionStart?;
    private selectionEnd?;
    constructor(elementRef: ElementRef, tokenService: TokenizerService);
    ngOnInit(): void;
    ngOnChanges(changes: SimpleChanges): void;
    /**
     * Provides start and end index of the currently selected text.
     *
     * @returns Returns the start index and end index of the currently selected text range. Returns `undefined`
     * if no text is currently selected.
     */
    getCurrentTextSelection(): ISelection | undefined;
    /**
     * Checks whether the given text selection is overlapping with existing annotations.
     *
     * @param selection The current text selection.
     * @returns Returns `true` if the given text selection is (partially) overlapping with
     * an existing annotation. Returns `false` otherwise.
     */
    isOverlappingWithExistingAnnotations(selection: ISelection): boolean;
    private updateTextSelection;
    static ɵfac: i0.ɵɵFactoryDeclaration<NgxAnnotateTextComponent, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<NgxAnnotateTextComponent, "ngx-annotate-text", never, { "annotations": "annotations"; "annotationClass": "annotationClass"; "removable": "removable"; "text": "text"; }, { "annotationsChange": "annotationsChange"; }, never, never>;
}
